{
 "cells": [
  {
   "cell_type": "markdown",
   "id": "7da09ed5-4577-4e6b-b841-442e99d3c194",
   "metadata": {},
   "source": [
    "# Wifi"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "4c00b860-52ae-4210-825c-4b69a5783585",
   "metadata": {},
   "source": [
    "Raspberry Pi Pico\n",
    "\n",
    "* [Raspberry Pi Pico Pinout Diagram](https://datasheets.raspberrypi.com/pico/Pico-R3-A4-Pinout.pdf)\n",
    "\n",
    "\n",
    "ESP-01 and ESP8266\n",
    "\n",
    "* [ESP8266 Wikipedia article](https://en.wikipedia.org/wiki/ESP8266)\n",
    "\n",
    "* [ESP8266 Non-OS AT Instruction Set](https://www.espressif.com/sites/default/files/documentation/4a-esp8266_at_instruction_set_en.pdf) Espressif's documentation of the AT command set.\n",
    "\n",
    "* [ESP8266 AT Commands from Electronics Hub](https://www.electronicshub.org/esp8266-at-commands/) A tutorial summary of the AT command set.\n",
    "\n",
    "* [AT Command Reference](http://room-15.github.io/blog/2015/03/26/esp8266-at-command-reference/) A blog description of all known AT commands implemented in the default firmware of the ESP8266 mcu.\n",
    "\n",
    "Examples\n",
    "* [Webserver Using Pi Pico and ESP01](https://www.instructables.com/Webserver-Using-Pi-Pico-and-ESP01/)\n",
    "\n",
    "* [Connect ESP-01S (ESP8266) to Raspberry Pi Pico/MicroPython, using AT Command](http://helloraspberrypi.blogspot.com/2021/02/connect-esp-01s-esp8266-to-raspberry-pi.html)\n",
    "\n",
    "* [Using AT commands on the ESP8266](https://learn.circuit.rocks/using-at-commands-on-the-esp8266) \n",
    "\n",
    "* [Pico/MicroPython + ESP-01S (AT Command) act as TCP Client connect to Raspberry Pi/Python TCP Server](http://helloraspberrypi.blogspot.com/2021/02/picomicropython-esp-01s-at-command-act.html) A bare bones example of communicating with the ESP8266 using AT commands.\n",
    "\n",
    "* [Raspberry Pi Pico Micropython ESP8266 Library](https://github.com/noyelseth/rpi-pico-micropython-esp8266-lib) MicroPython classes to access ESP8266 by AT commands, and to parse HTTP response \n",
    "after an HTTP Post/Get operation.\n",
    "\n",
    "Video\n",
    "* [Raspberry Pi Pico gets WiFi via an ESP01 and UART](https://youtu.be/02yfpA3h_w8https://youtu.be/02yfpA3h_w8)"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "752da5c0-d7ee-4e37-9dab-71af2b7bb429",
   "metadata": {},
   "source": [
    "## ESP01 class"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 165,
   "id": "7e2886fb-fbd8-4ce7-9ace-dd679cd9e446",
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "serial exception on close write failed: [Errno 6] Device not configured\n",
      "Found serial ports: /dev/cu.usbmodem14401, /dev/cu.Bluetooth-Incoming-Port \n",
      "\u001b[34mConnecting to --port=/dev/cu.usbmodem14401 --baud=115200 \u001b[0m\n",
      "\u001b[34mReady.\n",
      "\u001b[0m"
     ]
    }
   ],
   "source": [
    "%serialconnect\n",
    "\n",
    "from machine import UART, Pin\n",
    "import time\n",
    "\n",
    "UART_DELAY_MS = 500\n",
    "\n",
    "class ESP_Error(Exception):\n",
    "    \"\"\"Raise custom exceptions for ESP module.\"\"\"\n",
    "    pass\n",
    "\n",
    "\n",
    "class ESP01():\n",
    "    \n",
    "    def __init__(self, uart_port=0, tx=16, rx=17, verbose=False):\n",
    "        self.txPin = Pin(tx)\n",
    "        self.rxPin = Pin(rx)\n",
    "        self.uart = UART(0, baudrate=115200, tx=self.txPin, rx=self.rxPin, timeout=10)\n",
    "        self.verbose = verbose\n",
    "        if self.verbose:\n",
    "            self.echo_on()\n",
    "            print(self.send(\"AT\"))\n",
    "        else:\n",
    "            self.echo_off()\n",
    "\n",
    "    def send(self, cmd, timeout=UART_DELAY_MS):\n",
    "        if self.verbose:\n",
    "            print(\"+\"*20 + \"\\n command:\\t\", cmd)\n",
    "            print(\" timeout:\\t\", timeout, \"ms\")\n",
    "\n",
    "        # write AT command\n",
    "        self.uart.write(cmd + \"\\r\\n\")\n",
    "        \n",
    "        # wait for data to appear in the buffer\n",
    "        toc = time.ticks_ms()\n",
    "        while time.ticks_ms() - toc < timeout:\n",
    "            if self.uart.any():\n",
    "                break\n",
    "                \n",
    "        # read buffer\n",
    "        rxData = bytes()\n",
    "        toc = time.ticks_ms()\n",
    "        while True:\n",
    "            if (time.ticks_ms() - toc) > timeout:\n",
    "                break\n",
    "            if self.uart.any():\n",
    "                rxData += self.uart.read(1)\n",
    "                toc = time.ticks_ms()\n",
    "                \n",
    "        # report response\n",
    "        try:\n",
    "            rxData = rxData.decode()\n",
    "        except UnicodeError:\n",
    "            if self.verbose:\n",
    "                print(\"Unicode error encountered, returning byte string.\")\n",
    "        if self.verbose:\n",
    "            print(\"response:\\n\", rxData)\n",
    "            print(\"-\"*20)\n",
    "            \n",
    "        if \"OK\" in rxData:\n",
    "            status = \"OK\"\n",
    "        elif \"FAIL\" in rxData:\n",
    "            status = \"FAIL\"\n",
    "        elif \"busy p...\" in rxData:\n",
    "            status = \"BUSY\"\n",
    "        elif \"ERROR\" in rxData:\n",
    "            status = \"ERROR\"\n",
    "        else:\n",
    "            status = \"UNKNOWN\"\n",
    "            \n",
    "        if status != \"OK\":\n",
    "            print(\"status = \", status)\n",
    "            \n",
    "        return rxData\n",
    "            \n",
    "        \n",
    "    def echo_off(self):\n",
    "        \"\"\"Turn off the echo feature of the ESP-01.\"\"\"\n",
    "        return self.send(\"ATE0\")\n",
    "    \n",
    "    def echo_on(self):\n",
    "        \"\"\"Turn on the echo feature of the ESP-01.\"\"\"\n",
    "        return self.send(\"ATE1\")\n",
    "    \n",
    "    def restart(self):\n",
    "        \"\"\"Restart ESP8266 module.\"\"\"\n",
    "        return self.send(\"AT+RST\", 500)\n",
    "    \n",
    "    def get_version(self):\n",
    "        \"\"\"Return firmware version.\"\"\"\n",
    "        return self.send(\"AT+GMR\")\n",
    "    \n",
    "    def set_wifi_mode(self, mode=1):\n",
    "        return self.send(\"AT+CWMODE_CUR=\" + str(mode))\n",
    "        \n",
    "    def get_wifi_mode(self):\n",
    "        response = self.send(\"AT+CWMODE_CUR?\")\n",
    "        if response is not None:\n",
    "            if \"1\" in response:\n",
    "                return \"STA\"\n",
    "            elif \"2\" in response:\n",
    "                return \"SoftAP\"\n",
    "            elif \"3\" in response:\n",
    "                return \"SoftAP +  STA\"\n",
    "            else:\n",
    "                raise ESP_Error(\"Unrecognized WiFi mode returned: \\n\" + response)\n",
    "        return None\n",
    "        \n",
    "    def get_APs(self):\n",
    "        return self.send(\"AT+CWLAP\", 5000)\n",
    "    \n",
    "    def connect(self, ssid, pwd):\n",
    "        cmd = \"AT+CWJAP=\" + '\"' + ssid + '\"' + ',' + '\"' + pwd + '\"'\n",
    "        return self.send(cmd.encode(), timeout=10000)\n",
    "    \n",
    "    def get_IP_address(self):\n",
    "        return self.send(\"AT+CIFSR\")\n",
    "    \n",
    "    def set_connection_mode(self, mode=1):\n",
    "        return self.send(\"AT+CIPMUX=1\")\n",
    "    \n",
    "    def webserver(self):\n",
    "        return self.send(\"AT+CIPSERVER=1,80\", 2000)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 166,
   "id": "75a0c53c-fb1c-420e-9688-b39c2ce2a9b5",
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "++++++++++++++++++++\n",
      " command:\t ATE1\n",
      " timeout:\t 500 ms\n",
      "Unicode error encountered, returning byte string.\n",
      "response:\n",
      " b'\\xfeATE1\\r\\r\\n\\r\\nOK\\r\\n'\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT\n",
      " timeout:\t 500 ms\n",
      "response:\n",
      " AT\n",
      "\n",
      "OK\n",
      "\n",
      "--------------------\n",
      "AT\n",
      "\n",
      "OK\n",
      "\n",
      "++++++++++++++++++++\n",
      " command:\t AT+RST\n",
      " timeout:\t 500 ms\n",
      "Unicode error encountered, returning byte string.\n",
      "response:\n",
      " b'AT+RST\\r\\r\\n\\r\\nOK\\r\\nWIFI DISCONNECT\\r\\n\\r\\n ets Jan  8 2013,rst cause:2, boot mode:(3,7)\\r\\n\\r\\nload 0x40100000, len 27728, room 16 \\r\\ntail 0\\r\\nchksum 0x2a\\r\\nload 0x3ffe8000, len 2124, room 8 \\r\\ntail 4\\r\\nchksum 0x07\\r\\nload 0x3ffe8850, len 9276, room 4 \\r\\ntail 8\\r\\nchksum 0xba\\r\\ncsum 0xba\\r\\n\\x8c\\xe3No\\x83l\\x8e\\x8cl\\x130\\xec\\x0c\\x88\\x80\\x0c\\x8es\\x88\\x03\\x8c\\xe3No\\x83l\\x8e\\x8fd\\x130\\xe4\\x0c\\x88\\x80\\x0c\\x8es\\x88\\x03\\x8c\\xe3No\\x83l\\x8e\\x98l\\x130\\xec\\x0c\\x88\\x80l\\x8es\\x88\\x03s\\x88\\x03\\x8f\\t\\x907\\xff\\r\\nready\\r\\n'\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+GMR\n",
      " timeout:\t 500 ms\n",
      "response:\n",
      " AT+GMR\n",
      "AT version:1.1.0.0(May 11 2016 18:09:56)\n",
      "SDK version:1.5.4(baaeaebb)\n",
      "compile time:May 20 2016 15:08:19\n",
      "OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CWMODE_CUR=1\n",
      " timeout:\t 500 ms\n",
      "response:\n",
      " AT+CWMODE_CUR=1\n",
      "\n",
      "OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t b'AT+CWJAP=\"ND-guest\",\"\"'\n",
      " timeout:\t 10000 ms\n",
      "...response:\n",
      " AT+CWJAP=\"ND-guest\",\"\"\n",
      "WIFI CONNECTED\n",
      "WIFI GOT IP\n",
      "\n",
      "OK\n",
      "\n",
      "--------------------\n",
      "AT+CWJAP=\"ND-guest\",\"\"\n",
      "WIFI CONNECTED\n",
      "WIFI GOT IP\n",
      "\n",
      "OK\n",
      "\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPMUX=1\n",
      " timeout:\t 500 ms\n",
      "response:\n",
      " AT+CIPMUX=1\n",
      "\n",
      "OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIFSR\n",
      " timeout:\t 500 ms\n",
      "response:\n",
      " AT+CIFSR\n",
      "+CIFSR:STAIP,\"10.5.244.171\"\n",
      "+CIFSR:STAMAC,\"84:cc:a8:b3:b6:f9\"\n",
      "\n",
      "OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSERVER=1,80\n",
      " timeout:\t 2000 ms\n",
      "response:\n",
      " AT+CIPSERVER=1,80\n",
      "\n",
      "OK\n",
      "\n",
      "--------------------\n",
      ".............AT response received\n",
      "0 : 0,CONNECT\n",
      "1 : \n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,17\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " +IPD,0,376:GET / HTTP/1.1\n",
      "Host: 10.5.244.171\n",
      "Upgrade-Insecure-Requests: 1\n",
      "Accept: text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8\n",
      "User-Agent: Mozilla/5.0 (iPhone; CPU iPhone OS 14_8 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) CriOS/94.0.4606.76 Mobile/15E148 Safari/604.1\n",
      "Accept-Language: en-us\n",
      "Accept-Encoding: gzip, deflate\n",
      "Connection: keep-alive\n",
      "\n",
      "AT+CIPSEND=0,17\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t HTTP/1.1 200 OK\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 17 bytes\n",
      "\n",
      "SEND OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,24\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,24\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t Content-type:text/html\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 24 bytes\n",
      "\n",
      "SEND OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,19\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,19\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t Connection: close\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 19 bytes\n",
      "\n",
      "SEND OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,2\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,2\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t \n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 2 bytes\n",
      "\n",
      "SEND OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,14\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,14\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t <html><body>\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 14 bytes\n",
      "\n",
      "SEND OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,23\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,23\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t <h1>Hello World!</h1>\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 23 bytes\n",
      "\n",
      "SEND OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,11\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,11\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t message 0\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 11 bytes\n",
      "\n",
      "SEND OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,16\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,16\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t </body></html>\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 16 bytes\n",
      "\n",
      "SEND OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPCLOSE=0\n",
      " timeout:\t 500 ms\n",
      ".response:\n",
      " AT+CIPCLOSE=0\n",
      "0,CLOSED\n",
      "\n",
      "OK\n",
      "0,CONNECT\n",
      "\n",
      "+IPD,0,301:GET /favicon.ico HTTP/1.1\n",
      "Host: 10.5.244.171\n",
      "Connection: keep-alive\n",
      "DNT: 1\n",
      "User-Agent: Mozilla/5.0 (iPhone; CPU iPhone OS 14_8 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) CriOS/94.0.4606.76 Mobile/15E148 Safari/604.1\n",
      "Accept-Encoding: gzip, deflate\n",
      "Accept-Language: en-US,en;q=0.9\n",
      "\n",
      "\n",
      "--------------------\n",
      ".AT response received\n",
      "0 : 1,CONNECT\n",
      "1 : \n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,17\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " +IPD,1,376:GET / HTTP/1.1\n",
      "Host: 10.5.244.171\n",
      "Upgrade-Insecure-Requests: 1\n",
      "Accept: text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8\n",
      "User-Agent: Mozilla/5.0 (iPhone; CPU iPhone OS 14_8 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) CriOS/94.0.4606.76 Mobile/15E148 Safari/604.1\n",
      "Accept-Language: en-us\n",
      "Accept-Encoding: gzip, deflate\n",
      "Connection: keep-alive\n",
      "\n",
      "AT+CIPSEND=0,17\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t HTTP/1.1 200 OK\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 17 bytes\n",
      "\n",
      "--------------------\n",
      "status =  UNKNOWN\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,24\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,24\n",
      "busy s...\n",
      "\n",
      "--------------------\n",
      "status =  UNKNOWN\n",
      "++++++++++++++++++++\n",
      " command:\t Content-type:text/html\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " Content-type:text/html\n",
      "busy s...\n",
      "\n",
      "--------------------\n",
      "status =  UNKNOWN\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,19\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,19\n",
      "busy s...\n",
      "\n",
      "--------------------\n",
      "status =  UNKNOWN\n",
      "++++++++++++++++++++\n",
      " command:\t Connection: close\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " Connection: close\n",
      "busy s...\n",
      "\n",
      "SEND OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,2\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,2\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t \n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 2 bytes\n",
      "\n",
      "--------------------\n",
      "status =  UNKNOWN\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,14\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,14\n",
      "busy s...\n",
      "\n",
      "--------------------\n",
      "status =  UNKNOWN\n",
      "++++++++++++++++++++\n",
      " command:\t <html><body>\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " <html><body>\n",
      "busy s...\n",
      "\n",
      "--------------------\n",
      "status =  UNKNOWN\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,23\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,23\n",
      "busy s...\n",
      "\n",
      "--------------------\n",
      "status =  UNKNOWN\n",
      "++++++++++++++++++++\n",
      " command:\t <h1>Hello World!</h1>\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " <h1>Hello World!</h1>\n",
      "busy s...\n",
      "\n",
      "--------------------\n",
      "status =  UNKNOWN\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,11\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,11\n",
      "busy s...\n",
      "\n",
      "--------------------\n",
      "status =  UNKNOWN\n",
      "++++++++++++++++++++\n",
      " command:\t message 1\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " message 1\n",
      "busy s...\n",
      "\n",
      "--------------------\n",
      "status =  UNKNOWN\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,16\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,16\n",
      "busy s...\n",
      "\n",
      "--------------------\n",
      "status =  UNKNOWN\n",
      "++++++++++++++++++++\n",
      " command:\t </body></html>\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " </body></html>\n",
      "busy s...\n",
      "\n",
      "--------------------\n",
      "status =  UNKNOWN\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPCLOSE=0\n",
      " timeout:\t 500 ms\n",
      "response:\n",
      " AT+CIPCLOSE=0\n",
      "busy s...\n",
      "\n",
      "--------------------\n",
      "status =  UNKNOWN\n",
      "AT response received\n",
      "0 : \n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,17\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " SEND OK\n",
      "AT+CIPSEND=0,17\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t HTTP/1.1 200 OK\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 17 bytes\n",
      "\n",
      "--------------------\n",
      "status =  UNKNOWN\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,24\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,24\n",
      "busy s...\n",
      "\n",
      "--------------------\n",
      "status =  UNKNOWN\n",
      "++++++++++++++++++++\n",
      " command:\t Content-type:text/html\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " Content-type:text/html\n",
      "busy s...\n",
      "\n",
      "--------------------\n",
      "status =  UNKNOWN\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,19\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,19\n",
      "busy s...\n",
      "\n",
      "SEND OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t Connection: close\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " Connection: close\n",
      "ERROR\n",
      "\n",
      "--------------------\n",
      "status =  ERROR\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,2\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,2\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t \n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 2 bytes\n",
      "\n",
      "SEND OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,14\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,14\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t <html><body>\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 14 bytes\n",
      "\n",
      "SEND OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,23\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,23\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t <h1>Hello World!</h1>\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 23 bytes\n",
      "\n",
      "SEND OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,11\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,11\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t message 2\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 11 bytes\n",
      "\n",
      "SEND OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,16\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,16\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t </body></html>\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 16 bytes\n",
      "\n",
      "SEND OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPCLOSE=0\n",
      " timeout:\t 500 ms\n",
      "response:\n",
      " AT+CIPCLOSE=0\n",
      "0,CLOSED\n",
      "\n",
      "OK\n",
      "\n",
      "--------------------\n",
      ".....AT response received\n",
      "0 : 1,CLOSED\n",
      "1 : 0,CONNECT\n",
      "2 : \n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,17\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " +IPD,0,376:GET / HTTP/1.1\n",
      "Host: 10.5.244.171\n",
      "Upgrade-Insecure-Requests: 1\n",
      "Accept: text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8\n",
      "User-Agent: Mozilla/5.0 (iPhone; CPU iPhone OS 14_8 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) CriOS/94.0.4606.76 Mobile/15E148 Safari/604.1\n",
      "Accept-Language: en-us\n",
      "Accept-Encoding: gzip, deflate\n",
      "Connection: keep-alive\n",
      "\n",
      "AT+CIPSEND=0,17\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t HTTP/1.1 200 OK\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 17 bytes\n",
      "\n",
      "SEND OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,24\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,24\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t Content-type:text/html\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 24 bytes\n",
      "\n",
      "SEND OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,19\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,19\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t Connection: close\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 19 bytes\n",
      "\n",
      "SEND OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,2\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,2\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t \n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 2 bytes\n",
      "\n",
      "SEND OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,14\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,14\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t <html><body>\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 14 bytes\n",
      "\n",
      "SEND OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,23\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,23\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t <h1>Hello World!</h1>\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 23 bytes\n",
      "\n",
      "SEND OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,11\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,11\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t message 3\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 11 bytes\n",
      "\n",
      "SEND OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,16\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,16\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t </body></html>\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 16 bytes\n",
      "\n",
      "SEND OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPCLOSE=0\n",
      " timeout:\t 500 ms\n",
      "response:\n",
      " AT+CIPCLOSE=0\n",
      "0,CLOSED\n",
      "\n",
      "OK\n",
      "0,CONNECT\n",
      "\n",
      "+IPD,0,301:GET /favicon.ico HTTP/1.1\n",
      "Host: 10.5.244.171\n",
      "Connection: keep-alive\n",
      "DNT: 1\n",
      "User-Agent: Mozilla/5.0 (iPhone; CPU iPhone OS 14_8 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) CriOS/94.0.4606.76 Mobile/15E148 Safari/604.1\n",
      "Accept-Encoding: gzip, deflate\n",
      "Accept-Language: en-US,en;q=0.9\n",
      "\n",
      "\n",
      "--------------------\n",
      "......\n",
      "**[ys] <class 'serial.serialutil.SerialException'>\n",
      "**[ys] read failed: [Errno 6] Device not configured\n",
      "\n",
      "\n",
      "**[ys] <class 'serial.serialutil.SerialException'>\n",
      "**[ys] read failed: [Errno 6] Device not configured\n",
      "\n"
     ]
    }
   ],
   "source": [
    "# startup\n",
    "\n",
    "esp = ESP01(verbose=True)\n",
    "esp.restart()\n",
    "esp.get_version()\n",
    "esp.set_wifi_mode(1)\n",
    "\n",
    "# connection\n",
    "#esp.connect(\"Wilder\\u2019s Edge\", \"FreyaWatson\")\n",
    "print(esp.connect(\"ND-guest\", \"\"))\n",
    "esp.set_connection_mode()\n",
    "esp.get_IP_address()\n",
    "\n",
    "# webserver\n",
    "\n",
    "def read_http_request():\n",
    "    req = \"\"\n",
    "    while True:\n",
    "        recv = bytes()\n",
    "        line = 0\n",
    "        if esp.uart.any():\n",
    "            recv = esp.uart.readline()\n",
    "        recv = recv.decode()\n",
    "        recv = recv.replace(\"\\r\\n\", \"\\n\")\n",
    "        if len(recv) == 1:\n",
    "            break\n",
    "        else:\n",
    "            if len(recv) > 0:\n",
    "                req += recv\n",
    "    \n",
    "    lines = req.split(\"\\n\")\n",
    "    if \"+IPD\" in lines[0]:\n",
    "        print(\"http request received\")\n",
    "    else:\n",
    "        print(\"AT response received\")\n",
    "    for k, line in enumerate(lines):\n",
    "         print(k, \":\", line)\n",
    "        \n",
    "        \n",
    "def send_line(msg):\n",
    "    esp.send(f\"AT+CIPSEND=0,{len(msg)+2}\", 10)\n",
    "    esp.send(msg, 10)\n",
    "    \n",
    "def send_http_response(n):\n",
    "    send_line(\"HTTP/1.1 200 OK\")\n",
    "    send_line(\"Content-type:text/html\")\n",
    "    send_line(\"Connection: close\")\n",
    "    send_line(\"\");\n",
    "    send_line(\"<html><body>\")\n",
    "    send_line(\"<h1>Hello World!</h1>\")\n",
    "    send_line(f\"message {n}\")\n",
    "    send_line(\"</body></html>\")\n",
    "    esp.send(\"AT+CIPCLOSE=0\")\n",
    "    \n",
    "esp.webserver()\n",
    "recv_buf = bytes()\n",
    "toc = time.ticks_ms()\n",
    "n = 0\n",
    "while True:\n",
    "    read_http_request()\n",
    "    send_http_response(n)\n",
    "    n += 1\n",
    "    "
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 124,
   "id": "7811915f-d4b6-412d-83ec-228c70b72e90",
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "++++++++++++++++++++\n",
      " command:\t ATE1\n",
      " timeout:\t 500 ms\n",
      "Unicode error encountered, returning byte string.\n",
      "response:\n",
      " b'\\xfeATE1\\r\\r\\n\\r\\nOK\\r\\n'\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT\n",
      " timeout:\t 500 ms\n",
      "response:\n",
      " AT\n",
      "\n",
      "OK\n",
      "\n",
      "--------------------\n",
      "AT\n",
      "\n",
      "OK\n",
      "\n",
      "++++++++++++++++++++\n",
      " command:\t AT+RST\n",
      " timeout:\t 500 ms\n",
      "Unicode error encountered, returning byte string.\n",
      "response:\n",
      " b'AT+RST\\r\\r\\n\\r\\nOK\\r\\nWIFI DISCONNECT\\r\\n\\r\\n ets Jan  8 2013,rst cause:2, boot mode:(3,7)\\r\\n\\r\\nload 0x40100000, len 27728, room 16 \\r\\ntail 0\\r\\nchksum 0x2a\\r\\nload 0x3ffe8000, len 2124, room 8 \\r\\ntail 4\\r\\nchksum 0x07\\r\\nload 0x3ffe8850, len 9276, room 4 \\r\\ntail 8\\r\\nchksum 0xba\\r\\ncsum 0xba\\r\\n\\x8c\\xe3No\\x83l\\x8e\\x8cd\\x130\\xe4\\x0c\\x88\\x80\\x0c\\x8es\\x88\\x03\\x8c\\xe3No\\x83d\\x8e\\x8fl\\x130\\xec\\x0c\\x88\\x80\\x0c\\x8es\\x88\\x03\\x8c\\xe3No\\x83l\\x8e\\x98l\\x130\\xec\\x0c\\x88\\x80l\\x8es\\x88\\x03s\\x88\\x03\\x8f\\t\\x907\\xff\\r\\nready\\r\\n'\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+GMR\n",
      " timeout:\t 500 ms\n",
      "response:\n",
      " AT+GMR\n",
      "AT version:1.1.0.0(May 11 2016 18:09:56)\n",
      "SDK version:1.5.4(baaeaebb)\n",
      "compile time:May 20 2016 15:08:19\n",
      "OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CWMODE_CUR=1\n",
      " timeout:\t 500 ms\n",
      "response:\n",
      " AT+CWMODE_CUR=1\n",
      "\n",
      "OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t b'AT+CWJAP=\"Wilder\\xe2\\x80\\x99s Edge\",\"FreyaWatson\"'\n",
      " timeout:\t 5000 ms\n",
      ".response:\n",
      " AT+CWJAP=\"Wilder’s Edge\",\"FreyaWatson\"\n",
      "WIFI CONNECTED\n",
      "WIFI GOT IP\n",
      "\n",
      "OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPMUX=1\n",
      " timeout:\t 500 ms\n",
      "response:\n",
      " AT+CIPMUX=1\n",
      "\n",
      "OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIFSR\n",
      " timeout:\t 500 ms\n",
      "response:\n",
      " AT+CIFSR\n",
      "+CIFSR:STAIP,\"192.168.86.235\"\n",
      "+CIFSR:STAMAC,\"84:cc:a8:b3:b6:f9\"\n",
      "\n",
      "OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSERVER=1,80\n",
      " timeout:\t 2000 ms\n",
      ".response:\n",
      " AT+CIPSERVER=1,80\n",
      "\n",
      "OK\n",
      "\n",
      "--------------------\n",
      ".0,CONNECT\n",
      "\n",
      "\n",
      "\n",
      "+IPD,0,378:GET / HTTP/1.1\n",
      "\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,17\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " Host: 192.168.86.235\n",
      "Upgrade-Insecure-Requests: 1\n",
      "Accept: text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8\n",
      "User-Agent: Mozilla/5.0 (iPhone; CPU iPhone OS 14_8 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) CriOS/94.0.4606.76 MobiAT+CIPSEND=0,17\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t HTTP/1.1 200 OK\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 17 bytes\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,24\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,24\n",
      "busy s...\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t Content-type:text/html\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " Content-type:text/html\n",
      "busy s...\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,19\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,19\n",
      "busy s...\n",
      "\n",
      "SEND OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t Connection: close\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " Connection: close\n",
      "ERROR\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,2\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,2\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t \n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 2 bytes\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,23\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,23\n",
      "busy s...\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t <h1>Hello World!</h1>\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " <h1>Hello World!</h1>\n",
      "busy s...\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,22\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "SEND OK\n",
      "AT+CIPSEND=0,22\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t time = 7.459 seconds\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 22 bytes\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPCLOSE=0\n",
      " timeout:\t 500 ms\n",
      "response:\n",
      " \n",
      "SEND OK\n",
      "AT+CIPCLOSE=0\n",
      "0,CLOSED\n",
      "\n",
      "OK\n",
      "0,CONNECT\n",
      "\n",
      "+IPD,0,303:GET /favicon.ico HTTP/1.1\n",
      "Host: 192.168.86.235\n",
      "Connection: keep-alive\n",
      "DNT: 1\n",
      "User-Agent: Mozilla/5.0 (iPhone; CPU iPhone OS 14_8 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) CriOS/94.0.4606.76 Mobile/15E148 Safari/604.1\n",
      "Accept-Encoding: gzip, deflate\n",
      "Accept-Language: en-US,en;q=0.9\n",
      "\n",
      "\n",
      "--------------------\n",
      ".1,CONNECT\n",
      "\n",
      "\n",
      "\n",
      "+IPD,1,378:GET / HTTP/1.1\n",
      "\n",
      ".++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,17\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " Host: 192.168.86.235\n",
      "Upgrade-Insecure-Requests: 1\n",
      "Accept: text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8\n",
      "User-Agent: Mozilla/5.0 (iPhone; CPU iPhone OS 14_8 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) CriOS/94.0.4606.76 MobiAT+CIPSEND=0,17\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t HTTP/1.1 200 OK\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 17 bytes\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,24\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,24\n",
      "busy s...\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t Content-type:text/html\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " Content-type:text/html\n",
      "busy s...\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,19\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,19\n",
      "busy s...\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t Connection: close\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " Connection: close\n",
      "busy s...\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,2\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,2\n",
      "busy s...\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t \n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "busy s...\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,23\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,23\n",
      "busy s...\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t <h1>Hello World!</h1>\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " <h1>Hello World!</h1>\n",
      "busy s...\n",
      "\n",
      "SEND OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,23\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,23\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t time = 15.632 seconds\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 23 bytes\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPCLOSE=0\n",
      " timeout:\t 500 ms\n",
      "response:\n",
      " AT+CIPCLOSE=0\n",
      "busy s...\n",
      "\n",
      "SEND OK\n",
      "\n",
      "--------------------\n",
      "..1,CLOSED\n",
      "\n",
      "1,CONNECT\n",
      "\n",
      "\n",
      "\n",
      "+IPD,1,378:GET / HTTP/1.1\n",
      "\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,17\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " Host: 192.168.86.235\n",
      "Upgrade-Insecure-Requests: 1\n",
      "Accept: text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8\n",
      "User-Agent: Mozilla/5.0 (iPhone; CPU iPhone OS 14_8 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) CriOS/94.0.4606.76 MobiAT+CIPSEND=0,17\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t HTTP/1.1 200 OK\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 17 bytes\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,24\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,24\n",
      "busy s...\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t Content-type:text/html\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " Content-type:text/html\n",
      "busy s...\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,19\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,19\n",
      "busy s...\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t Connection: close\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " Connection: close\n",
      "busy s...\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,2\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,2\n",
      "busy s...\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t \n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "busy s...\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,23\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,23\n",
      "busy s...\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t <h1>Hello World!</h1>\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " <h1>Hello World!</h1>\n",
      "busy s...\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,23\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,23\n",
      "busy s...\n",
      "\n",
      "SEND OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t time = 27.904 seconds\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " time = 27.904 seconds\n",
      "ERROR\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPCLOSE=0\n",
      " timeout:\t 500 ms\n",
      "response:\n",
      " AT+CIPCLOSE=0\n",
      "0,CLOSED\n",
      "\n",
      "OK\n",
      "\n",
      "--------------------\n",
      "..1,CLOSED\n",
      "\n",
      "0,CONNECT\n",
      "\n",
      "\n",
      "\n",
      "+IPD,0,378:GET / HTTP/1.1\n",
      "\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,17\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " Host: 192.168.86.235\n",
      "Upgrade-Insecure-Requests: 1\n",
      "Accept: text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8\n",
      "User-Agent: Mozilla/5.0 (iPhone; CPU iPhone OS 14_8 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) CriOS/94.0.4606.76 MobiAT+CIPSEND=0,17\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t HTTP/1.1 200 OK\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 17 bytes\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,24\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,24\n",
      "busy s...\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t Content-type:text/html\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " Content-type:text/html\n",
      "busy s...\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,19\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,19\n",
      "busy s...\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t Connection: close\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " Connection: close\n",
      "busy s...\n",
      "\n",
      "SEND OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,2\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,2\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t \n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 2 bytes\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,23\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "SEND OK\n",
      "AT+CIPSEND=0,23\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t <h1>Hello World!</h1>\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 23 bytes\n",
      "\n",
      "SEND OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,23\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,23\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t time = 41.042 seconds\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 23 bytes\n",
      "\n",
      "SEND OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPCLOSE=0\n",
      " timeout:\t 500 ms\n",
      "response:\n",
      " AT+CIPCLOSE=0\n",
      "0,CLOSED\n",
      "\n",
      "OK\n",
      "0,CONNECT\n",
      "\n",
      "+IPD,0,303:GET /favicon.ico HTTP/1.1\n",
      "Host: 192.168.86.235\n",
      "Connection: keep-alive\n",
      "DNT: 1\n",
      "User-Agent: Mozilla/5.0 (iPhone; CPU iPhone OS 14_8 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) CriOS/94.0.4606.76 Mobile/15E148 Safari/604.1\n",
      "Accept-Encoding: gzip, deflate\n",
      "Accept-Language: en-US,en;q=0.9\n",
      "\n",
      "\n",
      "--------------------\n",
      ".1,CONNECT\n",
      "\n",
      "\n",
      "\n",
      "+IPD,1,378:GET / HTTP/1.1\n",
      "\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,17\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " Host: 192.168.86.235\n",
      "Upgrade-Insecure-Requests: 1\n",
      "Accept: text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8\n",
      "User-Agent: Mozilla/5.0 (iPhone; CPU iPhone OS 14_8 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) CriOS/94.0.4606.76 MobiAT+CIPSEND=0,17\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t HTTP/1.1 200 OK\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 17 bytes\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,24\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,24\n",
      "busy s...\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t Content-type:text/html\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " Content-type:text/html\n",
      "busy s...\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,19\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,19\n",
      "busy s...\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t Connection: close\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "SEND OK\n",
      "Connection: close\n",
      "ERROR\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,2\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,2\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t \n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 2 bytes\n",
      "\n",
      "SEND OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,23\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,23\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t <h1>Hello World!</h1>\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 23 bytes\n",
      "\n",
      "SEND OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,22\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,22\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t time = 44.12 seconds\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 22 bytes\n",
      "\n",
      "SEND OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPCLOSE=0\n",
      " timeout:\t 500 ms\n",
      "response:\n",
      " AT+CIPCLOSE=0\n",
      "0,CLOSED\n",
      "\n",
      "OK\n",
      "\n",
      "--------------------\n",
      "....1,CLOSED\n",
      "\n",
      "0,CONNECT\n",
      "\n",
      "\n",
      "\n",
      "+IPD,0,378:GET / HTTP/1.1\n",
      "\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,17\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " Host: 192.168.86.235\n",
      "Upgrade-Insecure-Requests: 1\n",
      "Accept: text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8\n",
      "User-Agent: Mozilla/5.0 (iPhone; CPU iPhone OS 14_8 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) CriOS/94.0.4606.76 MobiAT+CIPSEND=0,17\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t HTTP/1.1 200 OK\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 17 bytes\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,24\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,24\n",
      "busy s...\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t Content-type:text/html\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " Content-type:text/html\n",
      "busy s...\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,19\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,19\n",
      "busy s...\n",
      "\n",
      "SEND OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t Connection: close\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " Connection: close\n",
      "ERROR\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,2\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,2\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t \n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 2 bytes\n",
      "\n",
      "SEND OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,23\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,23\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t <h1>Hello World!</h1>\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 23 bytes\n",
      "\n",
      "SEND OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,23\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,23\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t time = 68.918 seconds\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 23 bytes\n",
      "\n",
      "SEND OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPCLOSE=0\n",
      " timeout:\t 500 ms\n",
      "response:\n",
      " AT+CIPCLOSE=0\n",
      "0,CLOSED\n",
      "\n",
      "OK\n",
      "0,CONNECT\n",
      "\n",
      "+IPD,0,303:GET /favicon.ico HTTP/1.1\n",
      "Host: 192.168.86.235\n",
      "Connection: keep-alive\n",
      "DNT: 1\n",
      "User-Agent: Mozilla/5.0 (iPhone; CPU iPhone OS 14_8 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) CriOS/94.0.4606.76 Mobile/15E148 Safari/604.1\n",
      "Accept-Encoding: gzip, deflate\n",
      "Accept-Language: en-US,en;q=0.9\n",
      "\n",
      "\n",
      "--------------------\n",
      "...1,CONNECT\n",
      "\n",
      "\n",
      "\n",
      "+IPD,1,378:GET / HTTP/1.1\n",
      "\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,17\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " Host: 192.168.86.235\n",
      "Upgrade-Insecure-Requests: 1\n",
      "Accept: text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8\n",
      "User-Agent: Mozilla/5.0 (iPhone; CPU iPhone OS 14_8 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) CriOS/94.0.4606.76 MobiAT+CIPSEND=0,17\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t HTTP/1.1 200 OK\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 17 bytes\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,24\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "SEND OK\n",
      "AT+CIPSEND=0,24\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t Content-type:text/html\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 24 bytes\n",
      "\n",
      "SEND OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,19\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,19\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t Connection: close\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 19 bytes\n",
      "\n",
      "SEND OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,2\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,2\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t \n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 2 bytes\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,23\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "SEND OK\n",
      "AT+CIPSEND=0,23\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t <h1>Hello World!</h1>\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 23 bytes\n",
      "\n",
      "SEND OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,25\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,25\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t time = 84.27401 seconds\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 25 bytes\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPCLOSE=0\n",
      " timeout:\t 500 ms\n",
      "response:\n",
      " \n",
      "SEND OK\n",
      "AT+CIPCLOSE=0\n",
      "0,CLOSED\n",
      "\n",
      "OK\n",
      "\n",
      "--------------------\n",
      ".....0,CONNECT\n",
      "\n",
      "2,CONNECT\n",
      "\n",
      "\n",
      "\n",
      "+IPD,2,472:GET / HTTP/1.1\n",
      "\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,17\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " Host: 192.168.86.235\n",
      "Connection: keep-alive\n",
      "Cache-Control: max-age=0\n",
      "DNT: 1\n",
      "Upgrade-Insecure-Requests: 1\n",
      "User-Agent: Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/94.0.4606.81 Safari/537.36\n",
      "Accept: textAT+CIPSEND=0,17\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t HTTP/1.1 200 OK\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 17 bytes\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,24\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,24\n",
      "busy s...\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t Content-type:text/html\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " Content-type:text/html\n",
      "busy s...\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,19\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,19\n",
      "busy s...\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t Connection: close\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " Connection: close\n",
      "busy s...\n",
      "\n",
      "SEND OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,2\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,2\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t \n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 2 bytes\n",
      "\n",
      "SEND OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,23\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,23\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t <h1>Hello World!</h1>\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 23 bytes\n",
      "\n",
      "SEND OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,24\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,24\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t time = 109.225 seconds\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 24 bytes\n",
      "\n",
      "SEND OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPCLOSE=0\n",
      " timeout:\t 500 ms\n",
      "response:\n",
      " AT+CIPCLOSE=0\n",
      "0,CLOSED\n",
      "\n",
      "OK\n",
      "\n",
      "--------------------\n",
      "....1,CONNECT FAIL\n",
      "\n",
      "2,CLOSED\n",
      "\n",
      "..0,CONNECT\n",
      "\n",
      "\n",
      "\n",
      "+IPD,0,389:GET /favicon.ico HTTP/1.1\n",
      "\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,17\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " Host: 192.168.86.235\n",
      "Connection: keep-alive\n",
      "User-Agent: Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/94.0.4606.81 Safari/537.36\n",
      "DNT: 1\n",
      "Accept: image/avif,image/webp,image/apng,image/svg+xml,image/*,*/*;qAT+CIPSEND=0,17\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t HTTP/1.1 200 OK\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 17 bytes\n",
      "\n",
      "SEND OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,24\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,24\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t Content-type:text/html\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 24 bytes\n",
      "\n",
      "SEND OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,19\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,19\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t Connection: close\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 19 bytes\n",
      "\n",
      "SEND OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,2\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,2\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t \n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 2 bytes\n",
      "\n",
      "SEND OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,23\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,23\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t <h1>Hello World!</h1>\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 23 bytes\n",
      "\n",
      "SEND OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,23\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,23\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t time = 141.62 seconds\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 23 bytes\n",
      "\n",
      "SEND OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPCLOSE=0\n",
      " timeout:\t 500 ms\n",
      "response:\n",
      " AT+CIPCLOSE=0\n",
      "0,CLOSED\n",
      "\n",
      "OK\n",
      "\n",
      "--------------------\n",
      "...0,CONNECT\n",
      "\n",
      "\n",
      "\n",
      "+IPD,0,446:GET / HTTP/1.1\n",
      "\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,17\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " Host: 192.168.86.235\n",
      "Connection: keep-alive\n",
      "DNT: 1\n",
      "Upgrade-Insecure-Requests: 1\n",
      "User-Agent: Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/94.0.4606.81 Safari/537.36\n",
      "Accept: text/html,application/xhtml+xmAT+CIPSEND=0,17\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t HTTP/1.1 200 OK\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 17 bytes\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,24\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,24\n",
      "busy s...\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t Content-type:text/html\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " Content-type:text/html\n",
      "busy s...\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,19\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,19\n",
      "busy s...\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t Connection: close\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " Connection: close\n",
      "busy s...\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,2\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,2\n",
      "busy s...\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t \n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "busy s...\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,23\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "SEND OK\n",
      "AT+CIPSEND=0,23\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t <h1>Hello World!</h1>\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 23 bytes\n",
      "\n",
      "SEND OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,24\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,24\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t time = 156.518 seconds\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 24 bytes\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPCLOSE=0\n",
      " timeout:\t 500 ms\n",
      "response:\n",
      " \n",
      "SEND OK\n",
      "AT+CIPCLOSE=0\n",
      "0,CLOSED\n",
      "\n",
      "OK\n",
      "0,CONNECT\n",
      "\n",
      "+IPD,0,389:GET /favicon.ico HTTP/1.1\n",
      "Host: 192.168.86.235\n",
      "Connection: keep-alive\n",
      "User-Agent: Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/94.0.4606.81 Safari/537.36\n",
      "DNT: 1\n",
      "Accept: image/avif,image/webp,image/apng,image/svg+xml,image/*,*/*;q=0.8\n",
      "Referer: http://192.168.86.235/\n",
      "Accept-Encoding: gzip, deflate\n",
      "Accept-Language: en-US,en;q=0.9\n",
      "\n",
      "\n",
      "--------------------\n",
      ".1,CONNECT\n",
      "\n",
      "\n",
      "\n",
      "+IPD,1,472:GET / HTTP/1.1\n",
      "\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,17\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " Host: 192.168.86.235\n",
      "Connection: keep-alive\n",
      "Cache-Control: max-age=0\n",
      "DNT: 1\n",
      "Upgrade-Insecure-Requests: 1\n",
      "User-Agent: Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/94.0.4606.81 Safari/537.36\n",
      "Accept: textAT+CIPSEND=0,17\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t HTTP/1.1 200 OK\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 17 bytes\n",
      "\n",
      "SEND OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,24\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,24\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t Content-type:text/html\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 24 bytes\n",
      "\n",
      "SEND OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,19\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,19\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t Connection: close\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 19 bytes\n",
      "\n",
      "SEND OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,2\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,2\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t \n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 2 bytes\n",
      "\n",
      "SEND OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,23\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " AT+CIPSEND=0,23\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t <h1>Hello World!</h1>\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 23 bytes\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPSEND=0,23\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "SEND OK\n",
      "AT+CIPSEND=0,23\n",
      "\n",
      "OK\n",
      "> \n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t time = 163.19 seconds\n",
      " timeout:\t 10 ms\n",
      "response:\n",
      " \n",
      "Recv 23 bytes\n",
      "\n",
      "SEND OK\n",
      "\n",
      "--------------------\n",
      "++++++++++++++++++++\n",
      " command:\t AT+CIPCLOSE=0\n",
      " timeout:\t 500 ms\n",
      "response:\n",
      " AT+CIPCLOSE=0\n",
      "0,CLOSED\n",
      "\n",
      "OK\n",
      "\n",
      "--------------------\n",
      "..1,CLOSED\n",
      "\n",
      "...................................................\n",
      "**[ys] <class 'serial.serialutil.SerialException'>\n",
      "**[ys] read failed: [Errno 6] Device not configured\n",
      "\n",
      "\n",
      "**[ys] <class 'serial.serialutil.SerialException'>\n",
      "**[ys] read failed: [Errno 6] Device not configured\n",
      "\n"
     ]
    }
   ],
   "source": [
    "    if esp.uart.any():\n",
    "        recv_buf += esp.uart.readline()\n",
    "    recv = recv_buf.decode()\n",
    "    if len(recv) > 0:\n",
    "        print(recv)\n",
    "        recv_buf = bytes()\n",
    "        if \"+IPD\" in recv:\n",
    "            time.sleep(1)\n",
    "            sendline(\"HTTP/1.1 200 OK\")\n",
    "            sendline(\"Content-type:text/html\")\n",
    "            sendline(\"Connection: close\")\n",
    "            sendline(\"\")\n",
    "            sendline(\"<h1>Hello World!</h1>\")\n",
    "            sendline(f\"time = {(time.ticks_ms() - toc)/1000} seconds\")\n",
    "            esp.send(\"AT+CIPCLOSE=0\")\n",
    "            time.sleep(1)\n",
    "            \n",
    "            "
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 42,
   "id": "468cf2cd-9155-4c42-b82f-cde0c398e306",
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "--------------------\n",
      " command:\t AT+CWMODE_CUR?\n",
      " timeout:\t 500 ms\n",
      "response:\t AT+CWMODE_CUR?\n",
      "+CWMODE_CUR:1\n",
      "\n",
      "OK\n",
      "\n",
      "--------------------\n",
      "STA\n"
     ]
    }
   ],
   "source": [
    "print(esp.get_wifi_mode())"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 2,
   "id": "a83f8a89-04a2-459c-abca-6ebfc2c6191e",
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "Found serial ports: /dev/cu.usbmodem14101, /dev/cu.Bluetooth-Incoming-Port \n",
      "\u001b[34mConnecting to --port=/dev/cu.usbmodem14101 --baud=115200 \u001b[0m\n",
      "\u001b[34mReady.\n",
      "\u001b[0m--------------------\n",
      " command:\t ATE1\n",
      " timeout:\t 500 ms\n",
      "response:\t b'ATE1\\r\\r\\n\\r\\nOK\\r\\n'\n",
      "--------------------\n",
      "--------------------\n",
      " command:\t AT\n",
      " timeout:\t 500 ms\n",
      "response:\t b'AT\\r\\r\\n\\r\\nOK\\r\\n'\n",
      "--------------------\n",
      "AT\n",
      "\n",
      "OK\n",
      "\n",
      "--------------------\n",
      " command:\t AT+GMR\n",
      " timeout:\t 500 ms\n",
      "response:\t b'AT+GMR\\r\\r\\nAT version:1.1.0.0(May 11 2016 18:09:56)\\r\\nSDK version:1.5.4(baaeaebb)\\r\\ncompile time:May 20 2016 15:08:19\\r\\nOK\\r\\n'\n",
      "--------------------\n",
      "AT+GMR\n",
      "AT version:1.1.0.0(May 11 2016 18:09:56)\n",
      "SDK version:1.5.4(baaeaebb)\n",
      "compile time:May 20 2016 15:08:19\n",
      "OK\n",
      "\n",
      "--------------------\n",
      " command:\t AT+CWMODE_CUR=1\n",
      " timeout:\t 500 ms\n",
      "response:\t b'AT+CWMODE_CUR=1\\r\\r\\n\\r\\nOK\\r\\n'\n",
      "--------------------\n",
      "AT+CWMODE_CUR=1\n",
      "\n",
      "OK\n",
      "\n",
      "--------------------\n",
      " command:\t AT+CWMODE_CUR?\n",
      " timeout:\t 500 ms\n",
      "response:\t b'AT+CWMODE_CUR?\\r\\r\\n+CWMODE_CUR:1\\r\\n\\r\\nOK\\r\\n'\n",
      "--------------------\n",
      "STA\n",
      "--------------------\n",
      " command:\t AT+CWJAP?\n",
      " timeout:\t 500 ms\n",
      "response:\t b'AT+CWJAP?\\r\\r\\n+CWJAP:\"Wilder\\xe2\\x80\\x99s Edge\",\"70:3a:cb:da:fb:ae\",11,-55\\r\\n\\r\\nOK\\r\\n'\n",
      "--------------------\n",
      "AT+CWJAP?\n",
      "+CWJAP:\"Wilder’s Edge\",\"70:3a:cb:da:fb:ae\",11,-55\n",
      "\n",
      "OK\n",
      "\n",
      "AT+CWJAP=\"Wilder’s Edge\",\"FreyaWatson\"\n",
      "--------------------\n",
      " command:\t b'AT+CWJAP=\"Wilder\\xe2\\x80\\x99s Edge\",\"FreyaWatson\"'\n",
      " timeout:\t 10000 ms\n",
      "..response:\t b'AT+CWJAP=\"Wilder\\xe2\\x80\\x99s Edge\",\"FreyaWatson\"\\r\\r\\nWIFI DISCONNECT\\r\\n'\n",
      "--------------------\n",
      "AT+CWJAP=\"Wilder’s Edge\",\"FreyaWatson\"\n",
      "WIFI DISCONNECT\n",
      "\n"
     ]
    }
   ],
   "source": [
    "      \n",
    "esp = ESP01S(verbose=True)\n",
    "#print(esp.restart())\n",
    "print(esp.version())\n",
    "print(esp.set_wifi_mode(1))\n",
    "print(esp.get_wifi_mode())\n",
    "print(esp.send(\"AT+CWJAP?\"))\n",
    "\n",
    "print(esp.connect_wifi(\"Wilder\\u2019s Edge\", \"FreyaWatson\"))"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 4,
   "id": "dcc5aa2b-6acf-429c-8767-bcfd5cbf7a03",
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "--------------------\n",
      " command:\t AT+CWJAP?\n",
      " timeout:\t 500 ms\n",
      "response:\t b'AT+CWJAP?\\r\\r\\n+CWJAP:\"Wilder\\xe2\\x80\\x99s Edge\",\"70:3a:cb:db:02:db\",1,-85\\r\\n\\r\\nOK\\r\\n'\n",
      "--------------------\n",
      "AT+CWJAP?\n",
      "+CWJAP:\"Wilder’s Edge\",\"70:3a:cb:db:02:db\",1,-85\n",
      "\n",
      "OK\n",
      "\n"
     ]
    }
   ],
   "source": [
    "print(esp.send(\"AT+CWJAP?\"))"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 71,
   "id": "c37e0f36-a6dd-4020-9148-3ef7b22c9070",
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "--------------------\n",
      " command:\t AT+CWLAP\n",
      " timeout:\t 5000 ms\n",
      "..response:\t b'AT+CWLAP\\r\\r\\n+CWLAP:(3,\"Wilder\\xe2\\x80\\x99s Edge\",-66,\"7c:2e:bd:8a:18:61\",1,13,0)\\r\\n+CWLAP:(3,\"Wilder\\xe2\\x80\\x99s Edge\",-45,\"70:3a:cb:db:02:db\",1,18,0)\\r\\n+CWLAP:(3,\"bermuda10\",-84,\"70:3a:cb:b4:9b:85\",1,10,0)\\r\\n+CWLAP:(3,\"bermuda10\",-75,\"70:3a:cb:b4:95:e4\",1,18,0)\\r\\n+CWLAP:(3,\"bermuda10\",-90,\"44:07:0b:0f:13:4f\",1,18,0)\\r\\n+CWLAP:(3,\"Wilder\\xe2\\x80\\x99s Edge\",-50,\"7c:2e:bd:8a:13:d3\",6,13,0)\\r\\n+CWLAP:(3,\"Wilder\\xe2\\x80\\x99s Edge\",-66,\"70:3a:cb:db:06:f0\",6,13,0)\\r\\n+CWLAP:(3,\"ATT9ssH4wv_2GEXT\",-88,\"8c:3b:ad:1f:2c:fc\",10,0,0)\\r\\n+CWLAP:(3,\"Wilder\\xe2\\x80\\x99s Edge\",-81,\"7c:2e:bd:8a:21:51\",11,16,0)\\r\\n+CWLAP:(3,\"Wilder\\xe2\\x80\\x99s Edge\",-82,\"44:07:0b:06:e8:d6\",11,15,0)\\r\\n\\r\\nOK\\r\\n'\n",
      "--------------------\n"
     ]
    }
   ],
   "source": [
    "resp =  esp.get_wifi_APs() "
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 76,
   "id": "553a138a-d133-462e-b045-035bc1cee028",
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "['+CWLAP:(3,\"Wilder\\u2019s Edge\",-66,\"7c:2e:bd:8a:18:61\",1,13,0)', '+CWLAP:(3,\"Wilder\\u2019s Edge\",-45,\"70:3a:cb:db:02:db\",1,18,0)', '+CWLAP:(3,\"bermuda10\",-84,\"70:3a:cb:b4:9b:85\",1,10,0)', '+CWLAP:(3,\"bermuda10\",-75,\"70:3a:cb:b4:95:e4\",1,18,0)', '+CWLAP:(3,\"bermuda10\",-90,\"44:07:0b:0f:13:4f\",1,18,0)', '+CWLAP:(3,\"Wilder\\u2019s Edge\",-50,\"7c:2e:bd:8a:13:d3\",6,13,0)', '+CWLAP:(3,\"Wilder\\u2019s Edge\",-66,\"70:3a:cb:db:06:f0\",6,13,0)', '+CWLAP:(3,\"ATT9ssH4wv_2GEXT\",-88,\"8c:3b:ad:1f:2c:fc\",10,0,0)', '+CWLAP:(3,\"Wilder\\u2019s Edge\",-81,\"7c:2e:bd:8a:21:51\",11,16,0)', '+CWLAP:(3,\"Wilder\\u2019s Edge\",-82,\"44:07:0b:06:e8:d6\",11,15,0)']\n"
     ]
    }
   ],
   "source": [
    "aps = [r for r in resp.split(\"\\r\\n\") if \"+CWLAP:\" in r]\n",
    "print(aps)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "id": "6f172777-6bf8-422b-9b10-8c14e5387e61",
   "metadata": {},
   "outputs": [],
   "source": []
  }
 ],
 "metadata": {
  "kernelspec": {
   "display_name": "MicroPython - USB",
   "language": "micropython",
   "name": "micropython"
  },
  "language_info": {
   "codemirror_mode": "python",
   "file_extension": ".py",
   "mimetype": "text/python",
   "name": "micropython"
  },
  "toc-autonumbering": false,
  "toc-showcode": false,
  "toc-showmarkdowntxt": true
 },
 "nbformat": 4,
 "nbformat_minor": 5
}
